exports = module.exports = (function () {
    'use strict'

    let store = {
        storage: window.localStorage,
        session: {
            storage: window.sessionStorage
        }
    }

    function serialize(val) {
        return JSON.stringify(val)
    }

    function deserialize(val) {
        if (typeof val !== 'string') {
            return undefined
        }
        return JSON.parse(val)
    }

    const api = {
        set(key, val) {
            this.storage.setItem(key, serialize(val))
        },

        get(key) {
            let val = deserialize(this.storage.getItem(key))
            if (val === null) {
                return false
            } else {
                return val
            }
        },

        has(key) {
            return this.get(key) !== false
        },

        remove(key) {
            this.storage.removeItem(key)
        },

        clear() {
            this.storage.clear()
        },

        getAll() {
            let ret = {}
            this.forEach((key, val) => {
                ret[key] = val
            })
            return ret
        },

        forEach(callback) {
            for (let i = 0; i < this.storage.length; i++) {
                let key = this.storage.key(i)
                callback(key, this.get(key))
            }
        }
    }

    /* 拼接 */
    Object.assign(store, api)
    Object.assign(store.session, api)

    return store
})()